//29
#include "win.h"
#include <QAction>
#include <QIcon>
#include <QMenu>
#include <QMenuBar>
#include <QFileDialog>
#include <QFile>
#include <QMessageBox>
#include <QTextStream>
#include <QPainter>
#include <QToolBar>

Win::Win() :
	map( 0 )
{
	open = new QAction( QIcon( ":/ico/open.bmp" ), tr( "&Open" ), this );
	compute = new QAction( QIcon( ":/ico/compute.bmp" ), tr( "&Compute" ), this );
	QMenu *m = new QMenu( tr( "&File"  ) );
	m->addAction( open );
	m->addAction( compute );
	m->addSeparator();
	QAction *quit = new QAction( tr( "Quit" ), this );
	m->addAction( quit );
	menuBar()->addMenu( m );
	connect( quit, SIGNAL(triggered()), SLOT(close()) );
	connect( open, SIGNAL(triggered()), SLOT(load()) );
	setCentralWidget( d = new Draw() );
	QToolBar *t = new QToolBar( this );
	addToolBar( t );
	t->addAction( open );
	t->addAction( compute );
	compute->setEnabled( false );
	connect( compute, SIGNAL(triggered()), SLOT(comp()) );
	resize( 500, 500 );
}

Win::~ Win() {
	delete map;
	delete d;
}

void Win::load() {
	QString fn = QFileDialog::getOpenFileName( this, tr( "Select the map" ) );
	if( !fn.isEmpty() ) {
		QFile *f( 0 );
		QTextStream *s( 0 );
		try {
			f = new QFile( fn );
			if( !f->open( QIODevice::ReadOnly ) )
				throw 1;
			s = new QTextStream( f );
			delete map;
			map = new Map( s );
			d->setMap( map );
			compute->setEnabled( true );
		}
		catch( ... ) {
			QMessageBox::critical( this, tr( "Load a map" ), tr( "Failed to load a map" ), QMessageBox::Ok, QMessageBox::NoButton );
			compute->setEnabled( false );
		}
		delete s;
		delete f;
	}
}

Map::Map( QTextStream *s ) {
	width = 0;
	height = 0;
	bool loadedSize = false;
	while( !s->atEnd() ) {
		QString ln = s->readLine().simplified();
		if( !loadedSize && !ln.isEmpty() ) {
			width = ln.size();
			loadedSize = true;
		}
		if( !ln.isEmpty() ) {
			if( width != ln.size() )
				throw 2;
			height ++;
			QVector< Pos > l;
			for( int i = 0; i < ln.size(); ++i ) {
				Pos p;
				switch( ln [ i ].toAscii() ) {
					case ':':
						p.field = GoodRoad;
						break;
					case '.':
						p.field = Road;
						break;
					case 'X':
						p.field = Out;
						break;
					case 'S':
						p.field = Start;
						startX = i;
						startY = height - 1;
						break;
					case 'C':
						p.field = End;
						break;
					default:
						throw 3;
				}
				l << p;
			}
			pos << l;
		}
	}
}

Draw::Draw() :
	map( 0 )
{}

void Draw::setMap( Map *m ) {
	map = m;
	update();
}

void Draw::paintEvent( QPaintEvent *e ) {
	QPainter painter( this );
	painter.setBrush( QBrush( Qt::black ) );
	painter.drawRect( 0, 0, width(), height() );
	if( map ) {
		int sizeX = width() / map->width;
		int sizeY = height() / map->height;
		for( int i = 0; i < map->width; ++i ) {
			for( int j = 0; j < map->height; ++j ) {
				QColor cl;
				switch( map->pos[ j ][ i ].field ) {
					case Map::Road: 
						cl = Qt::yellow;
						break;
					case Map::GoodRoad:
						cl = Qt::green;
						break;
					case Map::Start:
						cl = Qt::white;
						break;
					case Map::End:
						cl = Qt::magenta;
						break;
					case Map::Out:
						cl = Qt::red;
						break;
					case Map::Mark:
						cl = Qt::blue;
						break;
				}
				painter.setBrush( QBrush( cl ) );
				painter.drawRect( i * sizeX, j * sizeY, sizeX, sizeY );
			}
		}
	}
}

void Win::comp() {
	map->compute();
	d->update();
	(new PathShow( map->list ) )->show();
	compute->setEnabled( false );
}

static const int moveX[] = { 1, 0, -1, 0, 1, 1, -1, -1 };
static const int moveY[] = { 0, 1, 0, -1, -1, 1, 1, -1 };

void Map::compute() {
	int posX = startX, posY = startY, speedX = 0, speedY = 0;
	while( pos[ posY ][ posX ].field != End ) {
		bool found = false;
		for( int k = 0; k < 8; ++k ) {
			int i = moveX[ k ], j = moveY[ k ];
			if( posX + i < 0 )
				continue;
			if( posX + i >= width ) 
				continue;
			if( posY + j < 0 ) 
				continue;
			if( posY  + j >= height )
				continue;
			if( ( pos[ posY + j ][ posX + i ].field == GoodRoad ) || pos[ posY + j ][posX + i ].field == End ) {
				if( ( ( i == speedX * -1 ) && ( i != 0 ) ) || ( j == speedY * -1 ) && ( j != 0 ) ) {
					insertSpeed( -1* speedX, -1*speedY );
					speedX = 0;
					speedY = 0;
				}
				insertSpeed( i-speedX, j-speedY );
				speedX = i;
				speedY = j;
				found = true;
				if( pos[ posY = posY + j ][ posX = posX + i ].field == GoodRoad )
					pos[ posY ][ posX ].field = Mark;
				break;
			}
		}
		if( !found )
			return;
	}
}

void Map::insertSpeed( int x, int y ) {
	list << QString( "%1,%2\n" ).arg( x ).arg( -1 * y );
}

void PathShow::on_saveButt_clicked() {
	QString fn = QFileDialog::getSaveFileName( this, tr( "Save the instructions" ) );
	if( !fn.isEmpty() ) {
		QFile *f = new QFile( fn );
		f->open( QIODevice::WriteOnly );
		QTextStream *s = new QTextStream( f );
		foreach( const QString &st, listS ) {
			(*s) << st << '\n';
		}
		delete s;
		delete f;
	}
}

#include "moc_win.cpp"
